@@ -455,11 +455,7 @@ TopicDisplayWidget::TopicDisplayWidget(
   tree_->setColumnCount(2);
 
   tree_->header()->setStretchLastSection(false);
-#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
-  tree_->header()->setResizeMode(0, QHeaderView::Stretch);
-#else
   tree_->header()->setSectionResizeMode(0, QHeaderView::Stretch);
-#endif
 
   enable_hidden_box_ = new QCheckBox("Show unvisualizable topics");
   enable_hidden_box_->setCheckState(Qt::Unchecked);
